import os
import shutil
import uvicore
from ohmyi3.app import App
from uvicore.support.dumper import dump, dd
from uvicore.exceptions import SmartException
from uvicore.console import command, argument, option

@command()
async def cli():
    """Initialize a stock ~/.config/ohmyi3/* configuration"""

    # Get the application instance
    app = App(must_exist=False);

    uvicore.log.header('Initializing ohmyi3')

    # Create ~/.config/ohmyi3/ and copy stubs
    if not os.path.exists(app.config_folder):
        uvicore.log.item(f"Creating {app.config_folder} folder")
        #os.mkdir(app.config_folder)
        uvicore.log.item(f"Copying base configs into {app.config_folder} folder")
        shutil.copytree(app.stubs_folder + '/', app.config_folder + '/')

    # Create ~/.config/i3
    if not os.path.exists(app.i3config_folder):
        uvicore.log.item(f"Creating {app.i3config_folder} folder")
        os.mkdir(app.i3config_folder)

    # Create ~/.config/i3status
    if not os.path.exists(app.i3status_folder):
        uvicore.log.item(f"Creating {app.i3status_folder} folder")
        os.mkdir(app.i3status_folder)

    uvicore.log.nl()
    uvicore.log('Done!  Now run: i3ctl generate')



from uvicore.configuration import env
from uvicore.typing import OrderedDict

# Running application configuration.
# This config only applies if this package is running as the main application.
# Accessible at config('app')

config = {
    'name': 'Ohmyi3 Configuration Manager',
    'main': 'ohmyi3',
    'debug': False,
    'packages': OrderedDict({
        'ohmyi3': {
            'provider': 'ohmyi3.services.ohmyi3.Ohmyi3',
        },
    }),
    'logger': {
        'console': {
            'enabled': True,
            'level': 'INFO',
            'colors': True,
        },
    },
}



import os
import platform
from uvicore.typing import Dict

# ohmyi3 configuration python class
# Configs as python gives unlimited flexibility and programability
# All of these self.* variables are accessible as jinja2 variables
# to dynamically control your config.d/* i3 files

class Settings(Dict):
    def __init__(self):

        # Desktop Environment (kde, xfce, i3, cinnamon, mate, gnome)
        self.de = 'kde'

        # Theme
        self.theme = 'manjaro'

        # Hostname
        self.hostname = platform.node()

        # Logged in User
        self.user = os.getlogin()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ohmyi3',
 'ohmyi3.commands',
 'ohmyi3.config',
 'ohmyi3.services',
 'ohmyi3.stubs']

package_data = \
{'': ['*'], 'ohmyi3.stubs': ['config.d/*']}

install_requires = \
['jinja2>=3.1.0,<3.2.0', 'uvicore==0.1.25']

entry_points = \
{'console_scripts': ['i3ctl = ohmyi3.commands.entrypoint:cli']}

setup_kwargs = {
    'name': 'ohmyi3',
    'version': '0.1.0',
    'description': '',
    'long_description': '# Ohmyi3\n\nDynamic i3 configuration manager\n',
    'author': 'Matthew Reschke',
    'author_email': 'mail@mreschke.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import uvicore
from ohmyi3.app import App
from uvicore.support.dumper import dump, dd
from uvicore.exceptions import SmartException
from uvicore.console import command, argument, option

@command()
async def cli():
    """Display this systems ohmyi3 configuration information"""

    # Get the application instance
    app = App();
    dd(app)



import os
import sys
import uvicore
import shutil
from glob import glob
from ohmyi3.app import App
from datetime import datetime
from uvicore.typing import Dict
from uvicore.support.dumper import dump, dd
from uvicore.exceptions import SmartException
from jinja2 import FileSystemLoader, Environment
from uvicore.console import command, argument, option

@command()
async def cli():
    """Dynamically Generate the i3 config from config.d/*"""

    # Get the application instance
    app = App()

    # Start the generation
    uvicore.log.header("Generating new i3 config using ohmyi3")

    # Fire off user defined before_generate_hook
    uvicore.log.item3("Firing user defined before_generate hook")
    await app.settings.before_generate()

    # If i3 config exists, back it up
    i3config_file = f"{app.i3config_folder}/config"
    if os.path.exists(i3config_file):
        now = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
        backup = os.path.realpath(f"{app.i3config_folder}/backup-{now}")
        uvicore.log.item(f"Backing up {i3config_file} to {backup}")
        shutil.copy(i3config_file, backup)

    # Get all config.d/* files
    os.chdir(app.configd_folder)
    files = sorted(glob("*.conf"))

    # Append config.d/* and theme.d/theme to new i3 config
    with open(i3config_file, "w") as f:
        # Loop and merge each config and append to
        for file in files:
            uvicore.log.item2(f"Appending {file}")
            f.write(template(app.configd_folder, file).render(**app.settings))
            f.write("\n\n\n")

        # Append the selected theme files
        theme_folder = f"{app.theme_folder}/{app.settings.theme}"
        theme_file = os.path.realpath(f"{theme_folder}/theme.conf")
        if os.path.exists(theme_file):
            uvicore.log.item(f"Appending THEME {app.settings.theme}")
            f.write(template(theme_folder, 'theme.conf').render(**app.settings))

    # Copy themed i3status or a default if no theme specific file exists
    i3status_folder = None
    if os.path.exists(f"{theme_folder}/i3status.conf"):
        # Use themed i3status.conf
        i3status_folder = f"{theme_folder}"
    elif os.path.exists(f"{app.theme_folder}/i3status.conf"):
        # Use default i3status.conf
        i3status_folder = f"{app.theme_folder}"
    if i3status_folder:
        uvicore.log.item(f"Copying {i3status_folder}/i3status.conf to {app.i3status_folder}/config")
        with open(f"{app.i3status_folder}/config", "w") as f:
            f.write(template(i3status_folder, 'i3status.conf').render(**app.settings))

    # Fire off user defined afer_generate_hook
    uvicore.log.item3("Firing user defined after_generate hook")
    await app.settings.after_generate()

    # Cleanup
    app._cleanup()

    # Done
    uvicore.log.nl()
    uvicore.log("Done!")
    uvicore.log(f"New {i3config_file} generated!")
    uvicore.log("Please reload i3!")


def template(path, file):
    loader = FileSystemLoader(searchpath=path)
    env = Environment(loader=loader)
    return env.get_template(file)




import os
import sys
import uvicore
from uvicore.console import group
from ohmyi3.services import bootstrap
from uvicore.support.module import load

# Bootstrap the Uvicore application from the console entrypoint
app = bootstrap.application(is_console=True)

# Define a new asyncclick group
@group()
def cli():
    """Ohmyi3 - The dynamic variable driven i3 configuration manager

    Copyright (c) 2023 Matthew Reschke License http://mreschke.com/license/mit
    """
    pass

# Dynamically add in all commands from this package matching this command_group
command_group='i3ctl'
package = uvicore.app.package(main=True);
if ('console' in package and package.registers.commands and uvicore.app.is_console):
    for key, group in package.console['groups'].items():
        if key == command_group:
            for command_name, command_class in group.commands.items():
                cli.add_command(load(command_class).object, command_name)

# Instantiate the asyncclick group
try:
    cli(_anyio_backend='asyncio')
except KeyboardInterrupt:
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)



import os
import sys
import shutil
import uvicore
from uvicore.typing import Dict
from uvicore.support.dumper import dump, dd

class App(Dict):

    def __init__(self, must_exist = True):
        # Ohmyi3 main folder (~/.config/ohmyi3)
        # Override with env OHMYI3_CONFIG_PATH
        self.config_folder = self.path(uvicore.config('ohmyi3.config_path'), must_exist,
            message="Perhpas you havent run 'i3ctl init' yet??")

        # config.d folder (~/.config/ohmyi3/config.d)
        self.configd_folder = self.path(self.config_folder + '/config.d', must_exist)

        # themes.d folder (~/.config/ohmyi3/themes.d)
        self.theme_folder = self.path(self.config_folder + '/themes.d')

        # i3 config path (~/.config/i3) and file
        # Override with env OHMYI3_I3CONFIG_PATH
        self.i3config_folder = self.path(uvicore.config('ohmyi3.i3config_path'))

        # i3status path (~/.config/i3status/config)
        self.i3status_folder = self.path(uvicore.config('ohmyi3.i3status_path'))

        # Stubs folder
        self.app_folder = self.path(uvicore.app.package(main=True).path, True)
        self.stubs_folder = self.path(self.app_folder + '/stubs', True)

        # Import user settings (~/.config/ohmyi3/settings.py)
        if must_exist:
            self.settings = self._import_settings();

    def path(self, location, must_exist=False, message=None):
        real = os.path.realpath(os.path.expanduser(location))
        if must_exist:
            if not os.path.exists(real):
                print(f"{real} not found")
                if message: print(message)
                exit(1)
        return real


    def _cleanup(self):
        pycache = self.path(self.config_folder + '/__pycache__')
        if os.path.exists(pycache):
            shutil.rmtree(pycache)


    def _import_settings(self):
        sys.path.append(os.path.realpath(self.config_folder))
        from settings import Settings
        return Settings(self)


